@@ -9,9 +9,9 @@ import numpy as np
 from itertools import repeat
 from pathos.multiprocessing import Pool
 import inspect
-from . import python_shared_memory
+#from . import python_shared_memory
 from collections import OrderedDict
-from . import posix_shared_memory
+#from . import posix_shared_memory
 
 
 SHARED_MEMORIES_IN_SESSION = []
@@ -102,16 +102,24 @@ def array_from_shared_memory(name, backend="shared_array"):
     if backend == "shared_array":
         return sa.attach(name)
     elif backend == "python":
+        assert False, "Not supported"
         return python_shared_memory.np_array_from_shared_memory(name)
     elif backend == "posix":
+        assert False, "Not supported"
         return posix_shared_memory.np_array_from_shared_memory(name)
-    elif backend == "file" or backend == "compressed_file":
+    elif backend == "file":
         return np.load("." + name + ".npy")
+    elif backend == "compressed_file":
+        return np.load("." + name + ".npz")["arr_0"]
     else:
         raise Exception("Invalid backend %s" % backend)
 
 
 def array_to_shared_memory(name, array, backend):
+    if name is None:
+        name = random_name()
+
+    assert isinstance(name, str)
     global SHARED_MEMORIES_IN_SESSION
     if backend == "shared_array":
         try:
@@ -127,14 +135,19 @@ def array_to_shared_memory(name, array, backend):
             raise
         SHARED_MEMORIES_IN_SESSION.append(name)
     elif backend == "python":
+        assert False, "Not supported"
         python_shared_memory.np_array_to_shared_memory(name, array)
     elif backend == "posix":
+        assert False, "Not supported"
         posix_shared_memory.np_array_to_shared_memory(name, array)
-    elif backend == "file" or backend == "compressed_file":
+    elif backend == "file":
         np.save("." + name + ".npy", array, allow_pickle=True)
+    elif backend == "compressed_file":
+        np.savez_compressed("." + name + ".npz", array, allow_pickle=True)
     else:
         raise Exception("Invalid backend %s" % backend)
 
+    return name
 
 
 def base_type_to_shared_memory(object, name):
@@ -160,11 +173,15 @@ def to_file(object, base_name=None, compress=False):
     return object_to_shared_memory(object, base_name, backend=backend)
 
 
+def random_name():
+    random_generator = random.Random()  # create new generator so seed does not affect
+    return str(random_generator.randint(0, 10e15))
+
+
 def object_to_shared_memory(object, base_name=None, backend="shared_array"):
     t = time.perf_counter()
     if base_name is None:
-        random_generator = random.Random()  # create new generator so seed does not affect
-        base_name = str(random_generator.randint(0, 10e15))
+        base_name = random_name()
 
     data_bundle = DataBundle(base_name, backend)
 
@@ -191,6 +208,14 @@ def _object_is_basetype(object):
     return False
 
 
+def _is_iterable(object):
+    try:
+        iter(object)
+        return True
+    except TypeError:
+        return False
+
+
 def _object_to_shared_memory(object, name, data_bundle, backend="shared_array"):
     """
     Function will be called recursively. Responsibility: Put object in shared memory,
@@ -212,6 +237,12 @@ def _object_to_shared_memory(object, name, data_bundle, backend="shared_array"):
     elif issubclass(dict, object.__class__):
         return (object.__class__, {key: _object_to_shared_memory(value, name + "-" + str(key), data_bundle, backend)
                                    for key, value in object.items()})
+    elif _is_iterable(object):
+        # if iterable and non of above, assume we can wrap in a tuple
+        return (object.__class__, tuple(_object_to_shared_memory(element, name + "-" + str(i), data_bundle, backend)
+                                   for i, element in enumerate(object)))
+        #return (object.__class__, _object_to_shared_memory(tuple(object), name + "-" + str(object.__class__.__name__),
+        #                                                   data_bundle, backend))
     else:
         # is an object with possibly children
         # go through children and put these in shared memory
@@ -222,9 +253,16 @@ def _object_to_shared_memory(object, name, data_bundle, backend="shared_array"):
             if not hasattr(object, variable_name):
                 variable_name = "_" + variable_name
                 if not (hasattr(object, variable_name)):
-                    logging.warning("Object %s has init argument %s, but no property with the same name. Ignoring" % (
-                    object, variable_name))
-                    continue
+                    # last attempt: Try finding private variables
+                    for possible_var in object.__dict__.keys():
+                        if "__" in possible_var:
+                            if possible_var.split("__")[1] == variable_name[1:]:
+                                variable_name = possible_var
+                                break
+                    if not (hasattr(object, variable_name)):
+                        logging.warning("Object %s has init argument %s, but no property with the same name. Ignoring" % (
+                        object, variable_name))
+                        continue
 
             variable_data = getattr(object, variable_name)
             child_shared_memory_name = name + "-" + variable_name
@@ -259,7 +297,6 @@ def object_from_shared_memory(name, backend="shared_array"):
 
 
 def _object_from_shared_memory(name, description, data_bundle, backend="shared_array"):
-    #print("object from shared memory %s, class %s, description %s" % (name, cls, description))
     object_type, children = description
 
     if children is None:
@@ -278,13 +315,13 @@ def _object_from_shared_memory(name, description, data_bundle, backend="shared_a
     else:
         # has children
         if issubclass(list, object_type) or issubclass(tuple, object_type):
+            #print("      TUple with children: %s" % children)
             return object_type([_object_from_shared_memory(name + "-" + str(i), e, data_bundle, backend)
                     for i, e in enumerate(children)])
         elif issubclass(dict, object_type):
             return object_type((key_name, _object_from_shared_memory(name + "-" + str(key_name), child_desc, data_bundle, backend))
                                 for key_name, child_desc in children.items())
         else:
-            #print("Has children: %s" % str((description)))
             data = []
             for child_name, child_description in children.items():
                 shared_memory_name = name + "-" + child_name
@@ -352,29 +389,38 @@ def remove_shared_memory(name):
         if m.startswith(name + "__") or m.startswith(name + "-") or m == name:
             sa.delete(m)
             n_deleted += 1
+            if m in SHARED_MEMORIES_IN_SESSION:
+                SHARED_MEMORIES_IN_SESSION.remove(m)
 
-    if n_deleted == 0:
-        logging.warning("No shared memory with name %s" % name)
-        logging.warning("Available shared memories: %s" % shared_memories)
+    # if object contained no numpy arrays, no shared memory"
+    # was needed, but we may still have a file
+    if "__" not in name:
+        # not a subname
+        if name in SHARED_MEMORIES_IN_SESSION:
+            SHARED_MEMORIES_IN_SESSION.remove(name)
+        file = name + ".npz"
+        if os.path.exists(file):
+            os.remove(file)
+
+        if name in TMP_FILES_IN_SESSION:
+            TMP_FILES_IN_SESSION.remove(name)
+
+        n_deleted += 1
 
 
 def remove_shared_memory_in_session():
     for name in SHARED_MEMORIES_IN_SESSION:
-        try:
-            sa.delete(name)
-        except FileNotFoundError:
-            pass
-            #logging.warning("Tried to deleted shared memory %s that did not exist" % name)
+        remove_shared_memory(name)
 
-    for file in TMP_FILES_IN_SESSION:
-        if os.path.exists(file):
-            os.remove(file)
-
-    python_shared_memory.free_memory()
 
 def remove_all_shared_memory():
+    global SHARED_MEMORIES_IN_SESSION
     for shared in sa.list():
-        sa.delete(shared.name.decode("utf-8"))
+        name = shared.name.decode("utf-8")
+        sa.delete(name)
+        if name in SHARED_MEMORIES_IN_SESSION:
+            SHARED_MEMORIES_IN_SESSION.remove(name)
+
 
 def free_memory():
     remove_all_shared_memory()
@@ -406,6 +452,9 @@ def run_numpy_based_function_in_parallel(function, n_threads, arguments):
     # put each in shared memory
     # run function on chunk on separete thread, put in result array
     # return result array
+    #logging.warning("Bypassing paralell running")
+    #return function(*arguments)
+
     new_arguments = []
 
     # Put np arrays in shared memory, everything else we keep as is
